// Using two GPU's to increment by 1 an array of 4 integers,
// one GPU to increment the first two, the second GPU to increment the next two
// Don't need to use -pthread with nvcc

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

typedef struct argdata
{
  int deviceID;
  int *data;
} arg_data;

__global__ void kernel(int *data)
{
  data[threadIdx.x]++;
}
 
// Use 2 threads to increment 2 integers in an array
void *TaskCode(void *argument)
{
   arg_data *p;
   int *dev_data;
 
   p = (arg_data *) argument;
   cudaSetDevice(p->deviceID);
   cudaMalloc((void **) &dev_data, 2*sizeof(int));
   cudaMemcpy(dev_data, p->data, 2*sizeof(int), cudaMemcpyHostToDevice);
   kernel<<<1,2>>>(dev_data);
   cudaMemcpy(p->data, dev_data, 2*sizeof(int), cudaMemcpyDeviceToHost);
   cudaFree(dev_data);
   
   return NULL;
}
 
int main ()
{
   pthread_t thread1,thread2;
   arg_data arg1, arg2; 
   int a[4];

   a[0] = 0; a[1] = 1; a[2] = 2; a[3] = 3;
   arg1.deviceID = 0;
   arg2.deviceID = 1;
   arg1.data = &a[0];	// Address of first 2 ints
   arg2.data = &a[2];	// Address of second 2 ints

   /* create two threads */
   pthread_create(&thread1, NULL, TaskCode, (void *) &arg1);
   pthread_create(&thread2, NULL, TaskCode, (void *) &arg2);
 
   /* wait for all threads to complete */
   pthread_join(thread1, NULL);
   pthread_join(thread2, NULL);
 
   for (int i=0; i < 4; i++)
	printf("%d ", a[i]);
   printf("\n");
   return 0;
}
